classdef ConnexionRob < handle
    properties
        clientNb         %total number of clients % nbClient;
        clientCur        %current client % urbiClient;
        urbiClient
        clientList       %list of urbi clients % clientList;
        
        urbiIpList       % list of ip adresses of urbi servers % ipAddress;
        urbiPortList     %list of urbi ports. This list gives the nb of clients %port0;
        
        vrepIpList       % list of ip of the vrep simulators
        vrepInstanceList % list of instances used by vrep. %instance0;
        
        scenefile;
    end
    
    methods
        function obj = ConnexionRob(urbiIp, urbiPort, vrepIp, vrepInstance, sFile)
            % class constructor
            % urbiIp:   list of ip adresses of the urbi servers.
            %           If singleton: all urbi servers have the same ip
            % urbiPort: list of urbi ports. This list gives the nb of clients
            % vrepIp  : list of ip adresses of vrep simulators.
            %           If singleton, onely 1 simulator for all instances
            % vrepInstance: list of vrep simulators.
            %               if singleton, use instances vrepInstance,
            %               vrepInstance+1, vrepInstance+2...
            if(nargin == 5)
                
                if ~isempty(urbiPort)
                    obj.urbiPortList = urbiPort(:);
                    obj.clientNb     = length(urbiPort);
                end
                
                nUI = size(urbiIp,1)
                if (nUI == obj.clientNb)
                    obj.urbiIpList = urbiIp;
                elseif (nUI == 1)
                    obj.urbiIpList = repmat(urbiIp,obj.clientNb,1);
                elseif(nUI == 0)
                    obj.urbiIpList = repmat('127.0.0.1', obj.clientNb, 1);
                else
                    error('CONNEXIONROB: constructor size of urbiIpList incompatible')
                end
                
                
                nVI = size(vrepIp,1);
                if (nVI == obj.clientNb)
                    obj.vrepIpList = vrepIp;
                elseif (nVI == 1)
                    obj.vrepIpList = repmat(vrepIp,obj.clientNb,1);
                else
                    error('CONNEXIONROB: constructor size of vrepIpList incompatible')
                end
                
                nV = length(vrepInstance);
                if (nV == obj.clientNb)
                    obj.vrepInstanceList = vrepInstance(:);
                elseif (nV == 1)
                    obj.vrepInstanceList = [vrepInstance:(vrepInstance+obj.clientNb-1)]';
                elseif (nV == 0)
                    obj.vrepInstanceList = [1:obj.clientNb]';
                else
                    error('CONNEXIONROB: constructor size of vrepIinstanceList incompatible')
                end
                
                obj.scenefile     = sFile;
                obj.clientCur = 1;
            end
        end   %end function connexionRob
        
        
        function urbiClient = connectToVrep(obj, urbiIp, urbiPort, vrepIp, vrepInstance)
            urbiClient = [];
            urbiIp
            if isempty(urbiIp)
                urbiIp = {'127.0.0.1'}; %'192.168.214.142';
            end
            
            if isempty(urbiPort)
                urbiPort = 54000;
            end
            
            if isempty(vrepInstance)
                vrepInstance = 1;
            end
            
            try
                
                whos urbiIp
                urbiPort
                urbiClient = urbiConnect(urbiIp,urbiPort);
                urbiClearConnection(urbiClient);
                
                urbiSend(urbiClient, ['var vrepAddress = "',vrepIp,'";']) ;
                urbiSend(urbiClient, ['var vrepInstance = "',num2str(vrepInstance),'";']) ;
                urbiSendFile(urbiClient,'maiSimulation.u');
                urbiSendFile(urbiClient, 'FishingRod_Handle.u')
                urbiSendFile(urbiClient, 'FishingRod_Variables.u')
                urbiSendFile(urbiClient, 'FishingRod_SimuReg.u')
                urbiSendFile(urbiClient, 'FishingRod_Initialisation.u')
              
                  if obj.clientNb == 1
                      obj.clientList = [urbiClient];
                      obj.urbiClient = urbiClient;
                  end
            
            catch errorClient
                disp(errorClient.identifier);
            end
            
            % if we use only connectToVrep for 1 urbi client
          
            
        end %end function connectToVrep
        
        
        function obj = connectToPhysical(obj, portUrbi, instanceVrep)
                   
            if isempty(obj) || isempty(obj.urbiIpList)
                obj.urbiIpList = ['0.0.0.0'];
               % obj.urbiIpList = ['192.168.214.142'];
            end
            
            if isempty(portUrbi)
                portUrbi = 54000;
            end
            
            if isempty(instanceVrep)
                instanceVrep = 1;
            end
            
            rhobanAddress = '192.168.0.5';
            rhobanPort = 12345;
            
            try
                obj.urbiIpList(1)
                obj.urbiPortList = [portUrbi];
                obj.vrepInstanceList = [instanceVrep];
                
                urbiClient = urbiConnect(obj.urbiIpList(1),portUrbi);
                urbiClearConnection(urbiClient);
                
                urbiSendFile(urbiClient,'mai.u');
%                urbiSendFile(urbiClient,'maiPhysical.u');
                 urbiSendFile(urbiClient, 'FishingRod_Handle.u')
                 urbiSendFile(urbiClient, 'FishingRod_Variables.u')
                %                urbiSendFile(urbiClient, 'FishingRod_Initialisation.u')
                
                obj.urbiClient = urbiClient;
                
            catch errorClient
                disp(errorClient.identifier);
            end
            
        end %end function connectToPhysical
        
        
        function obj = connectSeveral(obj, fishingArm)
            for clientCur = 1:obj.clientNb
                obj.urbiIpList(clientCur,:)
                urbiClient = connectToVrep(obj, obj.urbiIpList(clientCur,:), obj.urbiPortList(clientCur), obj.vrepIpList(clientCur,:), obj.vrepInstanceList(clientCur) )
                obj.clientList = [obj.clientList; urbiClient];
                initialiseTimeUrbi(fishingArm, urbiClient);
            end
            
        end %end function connectSeveral
        
        %% switch to another instance of V-REP
        function obj = permuteInstanceVrep(obj)
            obj.clientCur = mod(obj.clientCur, obj.clientNb) +1;
            obj.urbiClient = obj.clientList(obj.clientCur);
        end %end function permuteInstanceVrep
        
        %% read the position of the object
        function outPos = readPosObj(obj)
            simEnd = 0;
            time = 0;
            TIME_LIM = 50;
            
            while(simEnd ==0 && time< TIME_LIM)
                pause(1);
                simEnd = urbiGetVariable(obj.urbiClient, 'simEnd')
                time = time+1;
                if ~isnumeric(simEnd)
                    simEnd = 0;
                end
            end
            
            outPos = urbiGetVariable(obj.urbiClient, 'pos')
        end %end function getPosObj
        
        %% read the position of the object and reinitialise
        function outPos = getPosObj(obj)
            simEnd = 0;
            time = 0;
            TIME_LIM = 50;
            
            while(simEnd ==0 && time< TIME_LIM)
                pause(0.1);
                simEnd = urbiGetVariable(obj.clientList(obj.clientCur), 'simEnd');
                time = time+1;
                if ~isnumeric(simEnd)
                    simEnd = 0;
                end
            end
            
            outPos = urbiGetVariable(obj.clientList(obj.clientCur), 'pos');
            %z = urbiGetVariable(obj.urbiClient, 'zpos'
            %urbiSendFile(obj.urbiClient, 'FishingRod_Initialisation.u')
            pause(0.5);
            urbiSendFile(obj.urbiClient, 'FishingRod_Initialisation.u')
            urbiSend(obj.urbiClient,'vrep.simStopSimulation();');
            
        end %end function getPosObj
        
        %% disconnect urbiClient when the object stabilises
        function outPos = waitDisconnect(obj)
            simEnd = 0;
            time = 0;
            TIME_LIM = 50;
            
            while(simEnd ==0 && time< TIME_LIM)
                pause(0.1);
                simEnd = urbiGetVariable(obj.clientList(obj.clientCur), 'simEnd');
                time = time+1;
                if ~isnumeric(simEnd)
                    simEnd = 0;
                end
            end
            
            outPos = urbiGetVariable(obj.clientList(obj.clientCur), 'pos');
            if ~iscell(outPos)
                outPos = urbiGetVariable(obj.clientList(obj.clientCur), 'vrep.simGetObjectPosition(hook,-1)');
            end
            
            disp('disconnect');
            urbiSend(obj.clientList(obj.clientCur),' vrep.simStopSimulation();')
            urbiSend(obj.clientList(obj.clientCur),' vrep.disconnect();')
            %pause(1);
            urbiDisconnect(obj.clientList(obj.clientCur));
            urbiSendFile(obj.clientList(obj.clientCur), 'FishingRod_Initialisation.u')
            
        end %end function waitDisconnect
        
        
    end %end methods
    
    
    methods(Static = true)
        function test_connectToVrep()
        path(path,'/Users/mai/Programmes/Urbi/liburbi-matlab/')
            path(path,'/Users/mai/Programmes/Urbi/liburbi-matlab/lib')
            path(path,'/Users/mai/Programmes/Urbi/liburbi-matlab/lib/helpers')
            path(path,'/Users/mai/Programmes/Urbi/liburbi-matlab/lib/helpers/tcp_udp_ip')
            
            
            saggGlobal
            variablesGlobales
            
            %if isempty(ipAd)
            urbiIp      = ['127.0.0.1'];
            %end
            urbiPort   = [51000 52000];
            vrepIp = ['193.50.110.97'];
            vrepInstance = [1];
            
            rFish     = FishingArm(6,4,0.01,3,0.3, WIN_PATH);
            connVrep  = ConnexionRob(urbiIp, urbiPort, vrepIp, vrepInstance, WIN_PATH);
            %urbiClient  = connectToVrep(connVrep,urbiIp(1,:), 51000, vrepIp(1,:), 1);   
            connectSeveral(connVrep, rFish)
            
        end
        
        
    end
    
end