classdef SimpleTest
    %change classdef name for your example
    
    % this class defines your environment, and how policies cause outcomes
    %
    % Copyright (c) 2013 Sao Mai Nguyen
    %               e-mail : nguyensmai@gmail.com
    %               INRIA Bordeaux - Sud-Ouest
    %               351 Cours de la Liberation, 33400, Talence,
    %               France,
    %               http://nguyensmai.free.fr/
    %
    
    
    %--------------------------------------------------------------------------
    %-------------------------  properties     --------------------------------
    %--------------------------------------------------------------------------
    
    properties
        nbOutcomeTypes;   %total number of task types
        plotenv ;    %boolean. true if you want to display the environment with drawEnvironment, else false
        context;
    end %end properties
    
    methods
        
        %--------------------------------------------------------------------------
        %--------------------------  constructor     ------------------------------
        %--------------------------------------------------------------------------
        
        function obj = SimpleTest()
            % constructor
            obj.nbOutcomeTypes = 1;
            obj.plotenv = 0;
        end %end of constructor
        
        
        
        function obj = drawEnvironment(obj)
            % displays the environment setup environment
            if(obj.plotenv)
                ;
            else
                ;
            end
        end  %end drawEnvironment
        
        function eval = evalPoints(obj, eval)
            % sets a set of benchmark points for the evaluation
            % eval : Evaluation object,where we edit its listEvalPointsD, listEvalPoints, listEvalWeight.
            %        each outcome type iOutcome \in [1 nbTaches], in this function we define:
            %        - nbTaches :  number of outcome types
            %        - listEvalPoints{iOutcome} a list of outcomes to be produced
            %        - listEvalWeight{iOutcome} a list of weights for each of the outcomes listEvalPoints{iOutcome}{i}
            %        - listEvalPointsD{iOutcome} is a cell of size 1x(dimesion of task subspace), listEvalPointsD{iOutcome}{iDim} is a vector of the (unique) values of listEvalPoints{iOutcome}(iDim)
            % cf file ../SGIMPlusieursTaches/Evaluation.m for details on the class Evaluation
            
            eval.nbOutcomeTypes       = 1;
            
            for iOutcome = 1:eval.nbOutcomeTypes
                for iDim =1:1 %for each dimension of the type of task iOutcome
                    eval.listEvalPointsD{iOutcome}{iDim} = [0:0.1:1]; % 1xn vector;
                end
                eval.listEvalPoints{iOutcome} = [0:0.1:1]'; %nxdimOutcome vector
                eval.listEvalWeight{iOutcome} = ones(size(eval.listEvalPoints{iOutcome})); %1xn vector
                
            end
        end
        
        
        
        %--------------------------------------------------------------------------
        %----- generating contexts and policies and checking their format   -------
        %--------------------------------------------------------------------------
        
        
        function obj = randomizeContext(obj)
            % generate random contexts
            obj.context = 30;
        end % end randomizeContext
        
        function param  = randParameters(obj)
            % generate random policy parameters
            param = 2*(rand(1)-0.5);
        end %end randomizeContext
        
        
        function param = normaliseParam(obj, param)
            % check that policy parameters are allowed
            % (within the right range, are in the proper format)
            
            param = min(max(param,-1),1);
        end %end normaliseParam
        
        function context = getContext(obj)
            context =  30;
        end %end getContext
        
        function obj = setContext(obj, context)
            obj.context = 30;
        end
        
        
        %--------------------------------------------------------------------------
        %-------- model of the environment: outcome reached by an outcome ---------
        %--------------------------------------------------------------------------
        
        function reached = executePolicy(obj,param, iOutcome)
            % simulate the environment change with a policy
            % param : policy parameter
            % iOutcome : type of outcome to read
            % reached : the induced/reached outcome
            reached = param^2;
%                 reached = param;
        end  %end execute
        
    end %end methods
    
    methods(Static=true)
        
        
        %--------------------------------------------------------------------------
        %------------------------  test functions ---------------------------------
        %--------------------------------------------------------------------------
        
        function test_EnvironmentSimulator()
            envSimu = EnvironmentSimulator()
            param = randParameters(envSimu)
            executePolicy(envSimu, param)
        end
        
    end %end static methods
    
end
