function Simulation( numberAct, numberIter,MODE, teach, fteach, filename)
%
% Copyright (c) 2012 Sao Mai Nguyen
%               e-mail : nguyensmai@gmail.com
%               INRIA Bordeaux - Sud-Ouest
%               351 Cours de la Liberation, 33400, Talence,
%               France, 
%               http://nguyensmai.free.fr/
%
% Simulation is one of the main program to launch an experiment.
     % It is called directly or by RunExperiment (which defines global variables and paths that need to be defined only once)
     % It resets all variables and calls fonction_principale to launch the real algorithm (in SGIMalgorithm folder)

saggGlobal
variablesGlobales
World_Initialize


if isempty(filename)
     fileMemo = ['../data/MemoTeach_',MODE,'_teacherid',num2str(teach),'_teacherf',num2str(fteach),'_date',date];
else
	  fileMemo = ['../data/',filename,'_',MODE,'teacherid',num2str(teach),'_teacherf',num2str(fteach),'_date',date];
end


%--------------------------------------------------------------------------
%------------------------  define global variables   ----------------------
%--------------------------------------------------------------------------
DimensionIn{1}   = 1
DimensionOut{1}  = 1
DimensionCont = 1;
numberAction  = numberAct;
numberIteration = numberIter;
nbOutcomeTypes = envSimu.nbOutcomeTypes;


Ymax{1} = [  1];
Ymin{1} = [ -1];

distYMax{1} = 2;



for iTache = 1:nbOutcomeTypes
    mapp{iTache} = Mapping(DimensionIn{iTache},DimensionOut{iTache}, DimensionCont, Ymin{iTache}, Ymax{iTache}, distYMax{iTache});
    maxDimY = max([maxDimY, DimensionOut{iTache}]);
    epmem{iTache} = EpisodicMemory(mapp{iTache},0.05, 0.05, 0.05); %distA, distY, gauss
end


evalPerf = Evaluation(envSimu);

%--------------------------------------------------------------------------
%------------------------  define different teachers ----------------------
%--------------------------------------------------------------------------

switch teach
    case 0
        hTeacher(1) = HumanTeacher(0, 0,1);
%        hTeacher(1) = HumanTeacher(0, 0, 'teacherRiacAirHockey1.mat');
        curLowLevel = LowLevel();
        curLowLevel.flagTeach = 0;
    case 1
        hTeacher(1) = HumanTeacher(1, fteach, 1,'simpleTeacher.mat');
        curLowLevel = LowLevel();
        curLowLevel.flagTeach = 0;
    otherwise
        disp('Unknown teacher');
end





%--------------------------------------------------------------------------
%  variables for the higher level exploration (outcome space and strategy)  
%--------------------------------------------------------------------------

global costStrat


SearchNear    = 0;
nbTr          = 1;
nbNN          = 10;                                                     % Nearest Neighbors
%     SearchNearTab = [];
NumReg        = 1;                                                      % ------Parameters
%predictedWorld = [];
meanErrors     = [];
pos_before     = [0 0 0 0 0 0];
cptHistMem     = 1;
histMem        = [];
Predictor      = [];
watchAround    = 0.1;
niter          = 1;
ProbMode1 = 0.7; %0.5; %0.7;
ProbMode3 = 0.25; %0.3; %0.2;

if strcmp(MODE,'SGIMSTRATWHO')
    ProbMode1 = 0.7; %0.5; %0.7;
    ProbMode3 = 0.25; %0.3; %0.2;
    nbStrategies = 3;
    costStrat    = [1 0.1 0.1];
    sample         = [10 5 5];
elseif  strcmp(MODE,'SGIMSTRAT')
    nbStrategies   = 7;
    costStrat      = [1 0.5 0.5 0.5 0.5 0.5 0.5];
    sample         = [10 4 4 4 4 4 4 ];
elseif  strcmp(MODE,'EMULATE')
    curLowLevel.nbImitate = fteach;
    nbStrategies   = 1;
    costStrat      = [1];
    sample         = [4];
elseif  strcmp(MODE,'REQUESTMIMIC')
    curLowLevel.nbImitate = fteach;
    nbStrategies   = 1;
    costStrat      = [1];
    sample         = [4];
elseif  strcmp(MODE,'RIAC')
     ProbMode1 = 0.7; %0.5; %0.7;
     ProbMode3 = 0.25; %0.3; %0.2;
    nbStrategies = 1;
    sample       = 10*ones(1,nbStrategies);
    costStrat    = ones(1,nbStrategies);
else
    nbStrategies = 1;
    sample       = 10*ones(1,nbStrategies);
    costStrat    = ones(1,nbStrategies);
end
CriterRegions = sample;

partReg = PartitionRegions(CriterRegions, sample, NumReg, mapp, nbStrategies, nbOutcomeTypes, watchAround)



%% Simulation
RandStream.setDefaultStream(RandStream('mt19937ar','seed',sum(100*clock)));
pause(1)


disp(['******************************************'])
disp(['Experiment of ' num2str(numberAction) ' Iterations'])
disp(['******************************************'])

fonction_principale(num2str(MODE));


disp(['******************************************'])
disp(['End of Experiment of ' num2str(numberAction) ' Iterations'])
disp(['******************************************'])
